from ctypes import *
from platform import *
import time

import pkg_resources
import os
import platform

from k2magic.dataframe_db_exception import DataFrameDBException

# 暂时写死，等待高总修复加载dll路径问题
# rdbLib = cdll.LoadLibrary('./ysdbLib.dll')
# rdbLib = cdll.LoadLibrary(r'C:\k2data_workspace\k2_magic\k2magic\ysdb\ysdbLib.dll')

# 遍历PATH环境变量加载ysdbLib.dll（不方便）
# import os
# dll_name = 'ysdbLib.dll'
# dll_path = None
# for path in os.environ['PATH'].split(os.pathsep):
#     dll_file = os.path.join(path, dll_name)
#     if os.path.exists(dll_file):
#         dll_path = dll_file
#         break
# if dll_path is None:
#     raise FileNotFoundError(f"Could not find {dll_name} in PATH")
# rdbLib = cdll.LoadLibrary(dll_path)

# 通过pkg_resources可以不依赖当前文件位置定位到所需dll文件
package_path = pkg_resources.resource_filename('k2magic', '')
if platform.system() == 'Windows':
	lib_name = 'ysdb/win/ysdbLib.dll'
elif platform.system() == 'Liunx':
	lib_name = 'ysdb/linux/libysdbLib.so'
elif platform.system() == 'Darwin':
	raise DataFrameDBException(f"YSDB not supported on platform {platform.system()} (macos)")
else:
	raise DataFrameDBException(f"YSDB not supported on platform {platform.system()}")

rdbLib = cdll.LoadLibrary(os.path.join(package_path, lib_name))

class PointData(Structure):
	_fields_ = [
		("tm", c_int),
		("ms", c_int),
		("fVal", c_double),
		("nVal", c_int),
		("state", c_ubyte),
		("flag", c_ubyte)
	]


class PointInfo(Structure):
	_fields_ = [
		("group", c_ubyte),
		("id", c_int),
		("tag", c_char*256),
		("desc", c_char*256),
		("type", c_int),
		("parentId", c_int)
	]

class FloatRealData(Structure):
	_fields_ = [
		("id", c_int),
		("tm", c_int),
		("ms", c_int),
		("val", c_float),
		("flag", c_ubyte),
		("state", c_ubyte)
	]

class FloatData(Structure):
	_fields_ = [
		("tm", c_int),
		("ms", c_int),
		("val", c_float),
		("flag", c_ubyte),
		("state", c_ubyte)
	]


class FloatStatData(Structure):
	_fields_ = [
		("maxVal", c_float),
		("maxTm", c_int),
		("maxMs", c_int),
		("minVal", c_float),
		("minTm", c_int),
		("minMs", c_int),
		("aveVal", c_float),
		("sumVal", c_float)
	]


class DoubleRealData(Structure):
	_fields_ = [
		("id", c_int),
		("tm", c_int),
		("ms", c_int),
		("val", c_double),
		("flag", c_ubyte),
		("state", c_ubyte)
	]


class DoubleData(Structure):
	_fields_ = [
		("tm", c_int),
		("ms", c_int),
		("val", c_double),
		("flag", c_ubyte),
		("state", c_ubyte)
	]


class DoubleStatData(Structure):
	_fields_ = [
		("maxVal", c_double),
		("maxTm", c_int),
		("maxMs", c_int),
		("minVal", c_double),
		("minTm", c_int),
		("minMs", c_int),
		("aveVal", c_double),
		("sumVal", c_double)
	]


class BoolRealData(Structure):
	_fields_ = [
		("id", c_int),
		("tm", c_int),
		("ms", c_int),
		("val", c_ubyte),
		("flag", c_ubyte)
	]


class BoolData(Structure):
	_fields_ = [
		("tm", c_int),
		("ms", c_int),
		("val", c_ubyte),
		("flag", c_ubyte)
	]


class IntRealData(Structure):
	_fields_ = [
		("id", c_int),
		("tm", c_int),
		("ms", c_int),
		("val", c_int),
		("state", c_ubyte),
		("flag", c_ubyte)
	]


class IntData(Structure):
	_fields_ = [
		("tm", c_int),
		("ms", c_int),
		("val", c_int),
		("state", c_ubyte),
		("flag", c_ubyte)
	]


class IntStatData(Structure):
	_fields_ = [
		("maxVal", c_int),
		("maxTm", c_int),
		("maxMs", c_int),
		("minVal", c_int),
		("minTm", c_int),
		("minMs", c_int),
		("aveVal", c_int),
		("sumVal", c_int)
	]


class LongRealData(Structure):
	_fields_ = [
		("id", c_int),
		("tm", c_int),
		("ms", c_int),
		("val", c_longlong),
		("flag", c_ubyte)
	]

class LongData(Structure):
	_fields_ = [
		("tm", c_int),
		("ms", c_int),
		("val", c_longlong),
		("flag", c_ubyte)
	]



class BlobData(Structure):
	_fields_ = [
		("tm", c_int),
		("ms", c_int),
		("val", POINTER(POINTER(c_ubyte))),
		("state", c_ubyte),
		("flag", c_ubyte)
	]


class BlobRealData(Structure):
	_fields_ = [
		("id", c_int),
		("tm", c_int),
		("ms", c_int),
		("val", POINTER(POINTER(c_ubyte))),
		("state", c_ubyte),
		("flag", c_ubyte)
	]


class BlobDataInfo(Structure):
	_fields_ = [
		("id", c_int),
		("tm", c_int),
		("ms", c_int),
		("size", c_int),
		("state", c_ubyte),
		("flag", c_ubyte)
	]



class PointRealData(Structure):
	_fields_ = [
		("group", c_ubyte),
		("id", c_int),
		("tm", c_int),
		("ms", c_int),
		("fVal", c_double),
		("nVal", c_int),
		("state", c_ubyte),
		("flag", c_ubyte)
	]


class PointData(Structure):
	_fields_ = [
		("tm", c_int),
		("ms", c_int),
		("fVal", c_double),
		("nVal", c_int),
		("state", c_ubyte),
		("flag", c_ubyte)
	]



class OneSecQuery(Structure):
	_fields_ = [
		("group", c_ubyte),
		("id", c_int),
		("tag", c_char*256),
		("tm", c_int),
		("ms", c_int),
		("mode", c_int),
		("interval", c_int)
	]


class HisQuery(Structure):
	_fields_ = [
		("group", c_ubyte),
		("id", c_int),
		("tag", c_char*256),
		("startTm", c_int),
		("startMs", c_int),
		("endTm", c_int),
		("endMs", c_int),
		("mode", c_int),
		("interval", c_int)
	]

class EventInfo(Structure):
	_fields_ = [
		("id", c_int),
		("sTm", c_int),
		("sMs", c_int),
		("type", c_int),
		("grade", c_ubyte),
		("content", c_char*512),
		
		("group", c_ubyte),
		("nVal", c_int),
		("fVal", c_double),
		("val1", c_int),
		("val2", c_int),
		("val3", c_int),
		("pars", c_char*512),
		
		("state", c_ubyte),
		("eTm", c_int),
		("eMs", c_int),
		("cTm", c_int),
		("cMs", c_int),
		("user", c_int)
	]


class BlobWaveHead(Structure):
	_fields_ = [
		("type", c_ubyte),
		("saveMode", c_ubyte),
		("working", c_ubyte),
		("periodType", c_ubyte),
		("periodFreq", c_uint),
		("periodCnt", c_uint),
		("pointCnt", c_uint),
		("start", c_ulonglong),
		("end", c_ulonglong),
		
		("maxVal", c_double),
		("minVal", c_double),
		
		("dataCnt", c_uint),
		("keyCnt", c_uint),
		
		("back", c_ubyte*512)
	]



rdbLib.rdbConnect.argtypes = [c_char_p, c_int]
rdbLib.rdbConnect.restype = c_int

rdbLib.readFloatRealDatasById.argtypes = c_int, POINTER(c_int), c_int, POINTER(POINTER(FloatRealData))
rdbLib.readFloatRealDatasById.restype = c_int

class RdbClient:
    mHandle = -1
    def connect(self, ip, port):
        ipStr = c_char_p(ip.encode('utf-8'))
        self.mHandle = rdbLib.rdbConnect(ipStr, c_int(port))
        return (self.mHandle > 0)
        
    def readFloatRealDatasById(self, idList):
        if self.mHandle <= 0:
            return None
        idCount = len(idList)
        idArr = (c_int*idCount)(*idList)
        retDatas = POINTER(FloatRealData)()
        
        dataCount = rdbLib.readFloatRealDatasById(self.mHandle, idArr, idCount, byref(retDatas))
        
        dataList = [retDatas[i] for i in range(dataCount)]
        return dataList
        
    def readBoolRealDatasById(self, idList):
        if self.mHandle <= 0:
            return None
        idCount = len(idList)
        idArr = (c_int*idCount)(*idList)
        retDatas = POINTER(BoolRealData)()
        
        dataCount = rdbLib.readBoolRealDatasById(self.mHandle, idArr, idCount, byref(retDatas))
        
        dataList = [retDatas[i] for i in range(dataCount)]
        return dataList
        
        
    def readIntRealDatasById(self, idList):
        if self.mHandle <= 0:
            return None
        idCount = len(idList)
        idArr = (c_int*idCount)(*idList)
        retDatas = POINTER(IntRealData)()
        
        dataCount = rdbLib.readIntRealDatasById(self.mHandle, idArr, idCount, byref(retDatas))
        
        dataList = [retDatas[i] for i in range(dataCount)]
        return dataList
        

    def readBoolHisData(self, group, id, tag, startTm, startMs, endTm, endMs, mode, interval):
        if self.mHandle <= 0:
            return None
        hisQuery = HisQuery(group, id, tag.encode(), startTm, startMs, endTm, endMs, mode, interval)

        retHisDatas = POINTER(BoolData)()
        hisDataCount = rdbLib.readBoolHisData(self.mHandle, byref(hisQuery), byref(retHisDatas))
        hisDataList = [retHisDatas[i] for i in range(hisDataCount)]
        return hisDataList
        
        
    def readIntHisData(self, group, id, tag, startTm, startMs, endTm, endMs, mode, interval):
        if self.mHandle <= 0:
            return None
        hisQuery = HisQuery(group, id, tag.encode(), startTm, startMs, endTm, endMs, mode, interval)

        retHisDatas = POINTER(IntData)()
        hisDataCount = rdbLib.readIntHisData(self.mHandle, byref(hisQuery), byref(retHisDatas))
        hisDataList = [retHisDatas[i] for i in range(hisDataCount)]
        return hisDataList
        
        
    def readFloatHisData(self, hisQuery):
        if self.mHandle <= 0:
            return None
        
        retHisDatas = POINTER(FloatData)()
        hisDataCount = rdbLib.readFloatHisData(self.mHandle, byref(hisQuery), byref(retHisDatas))
        hisDataList = [retHisDatas[i] for i in range(hisDataCount)]
        return hisDataList
        

    def readBoolHisDataByQueryParam(self, hisQuery):
        if self.mHandle <= 0:
            return None
        
        retHisDatas = POINTER(BoolData)()
        hisDataCount = rdbLib.readBoolHisData(self.mHandle, byref(hisQuery), byref(retHisDatas))
        hisDataList = [retHisDatas[i] for i in range(hisDataCount)]
        return hisDataList
        
        
    def readIntHisDataByQueryParam(self, hisQuery):
        if self.mHandle <= 0:
            return None
        
        retHisDatas = POINTER(IntData)()
        hisDataCount = rdbLib.readIntHisData(self.mHandle, byref(hisQuery), byref(retHisDatas))
        hisDataList = [retHisDatas[i] for i in range(hisDataCount)]
        return hisDataList
        
        
    def readBlobRealDatasById(self, idList):
        if self.mHandle <= 0:
            return None
           
        idCount = len(idList)
        idArr = (c_int*idCount)(*idList)
        retDatas = POINTER(BlobRealData)()
        
        dataCount = rdbLib.readBlobRealDatasById(self.mHandle, idArr, idCount, byref(retDatas))
        
        dataList = [retDatas[i] for i in range(dataCount)]
        return dataList
        
        
    def readBlobSecData(self, oneSecQuery):
        if self.mHandle <= 0:
            return None
           
        retHisDatas = POINTER(BlobRealData)()
        hisDataCount = rdbLib.readBlobSecData(self.mHandle, byref(oneSecQuery), byref(retHisDatas))
        hisDataList = [retHisDatas[i] for i in range(hisDataCount)]
        return hisDataList
        
        
        
    def readBlobHisInfo(self, hisQuery):
        if self.mHandle <= 0:
            return None
           
        retDatas = POINTER(BlobDataInfo)()
        
        dataCount = rdbLib.readBlobHisInfo(self.mHandle, hisQuery, byref(retDatas))
        dataList = [retDatas[i] for i in range(dataCount)]
        return dataList
        
        
        
    def readBlobHisData(self, blobDataInfo):
        if self.mHandle <= 0:
            return None
           
        retDatas = POINTER(BlobRealData)()        
        dataCount = rdbLib.readBlobHisInfo(self.mHandle, byref(blobDataInfo), byref(retDatas))
        return retDatas
        
    def readWaveData(self, oneSecQuery):
        if self.mHandle <= 0:
            return None
           
        retDatas = POINTER(BlobWaveData)()        
        dataCount = rdbLib.readBlobHisInfo(self.mHandle, byref(oneSecQuery), byref(retDatas))
        return retDatas
        
        
        
    def writeCtrlDataById(self, user, ctrlType, ctrlMode, realPointData):
        if self.mHandle <= 0:
            return None
        return rdbLib.writeCtrlDataById(self.mHandle, user, ctrlType, ctrlMode, realPointData)       
       
        
        
    def subscribePointRealData(self, callbackFunc):
        if self.mHandle <= 0:
            return None
    
        return rdbLib.subscribePointRealData(self.mHandle, callbackFunc)
        
        
    def evtConnect(self, port):
        if self.mHandle <= 0:
            return None
        return rdbLib.evtConnect(self.mHandle, port)
        
        
    def getPointIdAll(self, group):
        if self.mHandle <= 0:
            return None
        
        retDatas = POINTER(c_int)()
        count = rdbLib.getPointIdAll(self.mHandle, group, byref(retDatas))
        allIdList = [retDatas[i] for i in range(count)]
        return allIdList      
       
    def getPointInfos(self, group, idList):
        if self.mHandle <= 0:
            return None
        
        idCount = len(idList)
        idArr = (c_int*idCount)(*idList)
        retDatas = POINTER(PointInfo)()
        
        count = rdbLib.getPointInfos(self.mHandle, group, idArr, idCount, byref(retDatas))
        pointInfoList = [retDatas[i] for i in range(count)]
        return pointInfoList      
       
        
    def readPointHisData(self, hisQuery):
        if self.mHandle <= 0:
            return None
            
        retHisDatas = POINTER(PointData)()
        hisDataCount = rdbLib.readPointHisData(self.mHandle, byref(hisQuery), byref(retHisDatas))
        hisDataList = [retHisDatas[i] for i in range(hisDataCount)]
        return hisDataList
        
    #将hisQuery参数展开
    def readPointHisDataFlat(self, group, id, tag, startTm, startMs, endTm, endMs, mode, interval):
        if self.mHandle <= 0:
            return None
        hisQuery = HisQuery(group, id, tag.encode(), startTm, startMs, endTm, endMs, mode, interval)

        retHisDatas = POINTER(PointData)()
        hisDataCount = rdbLib.readPointHisData(self.mHandle, byref(hisQuery), byref(retHisDatas))
        hisDataList = [retHisDatas[i] for i in range(hisDataCount)]
        return hisDataList
        
        
    def readPointHisDatas(self, hisQueryList):
        if self.mHandle <= 0:
            return None
            
        hisResultList = [];
        for hisQuery in hisQueryList:
            hisRet = self.readPointHisData(hisQuery)
            hisResultList.append(hisRet)
        return hisResultList;
        
        
    def disconnect(self):
        rdbLib.rdbClose()

